// A program to display a surface of revolution
//
// Use 'l' to render the mesh
// Use 's' to render the surface
// Use 'e' to enable backface culling
// Use 'd' to disable backface culling

#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glut.h>
#include "Trackball.h"
#include "Lighting.h"
#include "Geometry.h"

const int windowWidth=400;
const int windowHeight=400;

typedef enum {LINE_MODE, SURFACE_MODE} RENDER_OPTION;
RENDER_OPTION current_render_option=SURFACE_MODE;

// material properties
GLfloat mat_specular[4];
GLfloat mat_ambient_and_diffuse[4];
GLfloat mat_shininess[1];

CTrackball trackball;
CLighting lighting;


void handleMouseMotion(int x, int y)
{	
	trackball.tbMotion(x, y);
}

void handleMouseClick(int button, int state, int x, int y)
{
	trackball.tbMouse(button, state, x, y);
}

void handleKeyboardEvent(unsigned char key, int x, int y)
{
	switch (key)
	{
		case 'l': current_render_option=LINE_MODE; 
					glutPostRedisplay(); break;
		case 's': current_render_option=SURFACE_MODE; 
					glutPostRedisplay(); break;
		case 'e': glCullFace(GL_BACK);
					glEnable(GL_CULL_FACE);
					glutPostRedisplay();  break;
		case 'd': glDisable(GL_CULL_FACE);
					glutPostRedisplay();  break;
		default: trackball.tbKeyboard(key);
	}
}

void display(void)
{
	glMatrixMode( GL_MODELVIEW );	// Set the view matrix ...
	glLoadIdentity();				// ... to identity.
    gluLookAt(0,0,6, 0,0,0, 0,1,0); // camera is on the z-axis
	trackball.tbMatrix();			// rotate scene using the trackball

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	const int numSegmentsS=10;
	const int numSegmentsT=32;

	glFrontFace(GL_CW);
	if (current_render_option==LINE_MODE)
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	else
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	int lt, lg, numLatitudes=16, numLongitudes=16;
	double x,y,z,r;
	for(lt=0; lt<numLatitudes-1; lt++)
	{
		glBegin(GL_QUAD_STRIP);
		for(lg=0; lg<=numLongitudes; lg++)
		{
			z = sin(Pi*lt/((double) (numLatitudes-1))-Pi/2.0f);
			r = cos(Pi*lt/((double) (numLatitudes-1))-Pi/2.0f);
			x = cos(2*Pi*lg/(double) numLongitudes)*r;
			y = sin(2*Pi*lg/(double) numLongitudes)*r;
	        glNormal3f(x,y,z);
			glVertex3f(x,y,z);
			z = sin(Pi*(lt+1)/((double) (numLatitudes-1))-Pi/2.0f);
			r = cos(Pi*(lt+1)/((double) (numLatitudes-1))-Pi/2.0f);
			x = cos(2*Pi*lg/(double) numLongitudes)*r;
			y = sin(2*Pi*lg/(double) numLongitudes)*r;
	        glNormal3f(x,y,z);
			glVertex3f(x,y,z);
		}	
		glEnd();
	}

	glFlush ();
	glutSwapBuffers();
}

void init(void) 
{
	// select clearing color (for glClear)
	glClearColor (1,1,1,1);	// RGB-value for black
	// enable depth buffering
	glEnable(GL_DEPTH_TEST);
	// initialize view (simple orthographic projection)
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(33,1,2,8);
	trackball.tbInit(GLUT_LEFT_BUTTON);

	// material properties 
	mat_ambient_and_diffuse[0]=0.8;		// red material ...
	mat_ambient_and_diffuse[1]=0.0;
	mat_ambient_and_diffuse[2]=0.0;
	mat_ambient_and_diffuse[3]=1;
	mat_specular[0]=0.8f;				// ... with white highlights
	mat_specular[1]=0.8f;				// if light source is reflected
	mat_specular[2]=0.8f;				// on the material surface.
	mat_specular[3]=1;
	mat_shininess[0]=100;
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_ambient_and_diffuse);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);

	// enable shading and lighting
	lighting.enable();
	glShadeModel(GL_SMOOTH);
}

void reshape(int width, int height ) {
	// Called at start, and whenever user resizes component
	int size = min(width, height);
	glViewport(0, 0, size, size);  // Largest possible square
	trackball.tbReshape(width, height);
}


// create a double buffered colour window
int main(int argc, char** argv)
{
	glutInit(&argc, argv);		
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(windowWidth, windowHeight); 
	glutInitWindowPosition(100, 100);
	glutCreateWindow("Surface of Revolution");
	init ();								// initialise view
	glutMouseFunc(handleMouseClick);		// Set function to handle mouse clicks
	glutMotionFunc(handleMouseMotion);		// Set function to handle mouse motion
	glutKeyboardFunc(handleKeyboardEvent);	// Set function to handle keyboard input
	glutDisplayFunc(display);		// Set function to draw scene
	glutReshapeFunc(reshape);		// Set function called if window gets resized
	glutMainLoop();
	return 0;
}
